@@ -6,6 +6,7 @@ class Structure(DesignationsObject):
     def __init__(self):
         super().__init__()
         self._endpoint = 'structures'
+        self._object_name = 'StructureDTO'
 
     
     def get_nodes(self, depth=1):
@@ -18,11 +19,11 @@ class Structure(DesignationsObject):
              if self.value is a list it will return a list of StructureNode objects.
         """
         if isinstance(self.value, dict):
-            nodes = StructureNode(self.get_value('id')).get(parameters={'depth':depth})
+            nodes = StructureNode(self.get_value('id')).get(parameters=[('depth', depth)])
         
         elif isinstance(self.value, list):
             nodes = [
-                StructureNode(structure_id).get(parameters={'depth':depth})
+                StructureNode(structure_id).get(parameters=[('depth', depth)])
                 for structure_id
                 in self.get_value('id')
             ]
@@ -37,4 +38,5 @@ class StructureNode(DesignationsObject):
         super().__init__()
         self.structure_id = structure_id
         self._endpoint = f'structures/{structure_id}/nodes'
+        self._object_name = 'StructureDTO'
 
