@@ -1,5 +1,6 @@
 import json
 from collections import defaultdict
+from copy import deepcopy
 
 import numpy as np
 import pandas as pd
@@ -36,14 +37,14 @@ class Object:
 
         keys = key.split('__')
 
-        if type(self.value) == dict:
+        if isinstance(self.value, dict):
             value = self.value
             for k in keys:
                 value = value[k]
         
-        elif type(self.value) == list:
+        elif isinstance(self.value, list):
             value = self.value
-            if index is not None:
+            if index:
                 value = value[index]
                 for k in keys:
                     value = value[k]
@@ -276,25 +277,8 @@ class Object:
                 temp[ks[-1]] = value
 
         else:
-            raise AIHE.AIHException(f'self.value and args must be both of type lsit or dict, got {type(self.value)} and {type(args)}')
+            raise AIHE.AIHException(f'self.value and args must be both of type list or dict, got {type(self.value)} and {type(args)}')
 
-
-        # if type(args) == list:            
-        #     for _args in args:
-        #         for arg, value in _args.items():
-        #             ks = arg.split(split)
-        #             temp = self.value
-        #             for k in ks[:-1]:
-        #                 temp = temp[k]
-        #             temp[ks[-1]] = value
-                    
-        # elif type(args) == dict:
-        #     for arg, value in args.items():
-        #         ks = arg.split(split)
-        #         temp = self.value
-        #         for k in ks[:-1]:
-        #             temp = temp[k]
-        #         temp[ks[-1]] = value
             
         
         """
@@ -416,7 +400,7 @@ class Object:
         """
     
         def implode_group(group):
-            new_val = group[col].to_list()
+            new_val = group[col].dropna().to_list()
             group = group.iloc[0]
             group[col] = new_val
             
@@ -450,12 +434,17 @@ class Object:
             
             # Explodes to have the list values on each line
             self_df = self_df.explode(self_key)
+            self_df_empty = self_df[self_df[self_key].isna()] # to be added afterwards
+            self_df = self_df[~self_df[self_key].isna()]
             other_df = other_df.set_index(other_key)
 
             # Expands each of the values with the information from the other dataframe
             self_df[self_key] = self_df[self_key].apply(lambda x : other_df.loc[x].to_dict())
 
-            # Implodes so the values that were split on each line is recombined into a list for the correct element
+            # Concatenate with the elements that had empty lists for the key
+            self_df = pd.concat([self_df, self_df_empty])
+
+            # Implodes so the values that were split on each line are recombined into a list for the correct element
             merged = self._implode(self_df, self_key)
 
         else:
@@ -488,7 +477,119 @@ class Object:
         __setstate__ is used to set the state of the object.
         It is used for pickle to be able to unpickle pickled objects.
 
-        IM: state - The value that should be the objects value
+        IN: state - The value that should be the objects value
         """
         self.__init__()
         self.value = state
+
+
+    def __str__(self):
+        """
+        __str__ is used to represent the object as a string.
+        """
+
+        s = (
+            f'{str(self.__class__).split()[1][1:-2].rsplit(".", 1)[0]}\n'
+            f'Length: {len(self)}'
+        )
+        return s
+
+
+    def __len__(self):
+        """
+        __len__ is used to return the length of the object.
+        In other words it returns the number objects that is represented in the self.value
+        """
+        
+        # if isinstance(self.value, type(None)):
+        #     return 0
+
+        if isinstance(self.value, dict):
+            return 1 if self.value else 0
+
+        elif isinstance(self.value, list):
+            return len(self.value)
+    
+
+    def __iter__(self):
+        """
+        __iter__ returns and iterator over the self.value
+        """
+
+        if isinstance(self.value, dict):
+            if self.value:
+                yield self.value
+
+        elif isinstance(self.value, list):
+            for elem in self.value:
+                yield elem
+        
+        else:
+            raise AIHE.AIHException(f'self.value is not of type list or dict but was {type(self.value)}')
+
+
+    def __add__(self, other):
+        """
+        __add__ is used to combine to objects of the same type.
+        It always returns the result with the self.value as a list.
+        The returned object is a new object and does not operate inplace. 
+        Use augmented assignmet to update self inplace.
+        """
+
+        # Check if both object are of same type
+        if type(self) != type(other):
+            type_self = str(self.__class__).split()[1][1:-2].rsplit(".", 1)[0]
+            type_other = str(other.__class__).split()[1][1:-2].rsplit(".", 1)[0]
+            raise AIHE.AIHException(f'Both objects must be of the same type. Got {type_self} and {type_other}')
+        
+        new_value = list(deepcopy(self))
+        other_new_value = list(deepcopy(other))
+
+        new_value.extend(other_new_value)
+        new_object = self.__class__().from_list(new_value)
+
+        return new_object
+
+    
+    def __iadd__(self, other):
+        """
+        __iadd__ is used to combine to objects of the same type inplace.
+        It always returns the result with the self.value as a list.
+        """
+        
+        new_object = self + other
+        self.value = new_object.value
+        return self
+
+    
+    def __sub__(self, other):
+        """
+        __sub__ is used to filter out the elements in the self.value that exists in other.value.
+        It always returns the result with the self.value as a list.
+        The returned object is a new object and does not operate inplace. 
+        Use augmented assignmet to update self inplace.
+        """
+
+        # Check if both object are of same type
+        if type(self) != type(other):
+            type_self = str(self.__class__).split()[1][1:-2].rsplit(".", 1)[0]
+            type_other = str(other.__class__).split()[1][1:-2].rsplit(".", 1)[0]
+            raise AIHE.AIHException(f'Both objects must be of the same type. Got {type_self} and {type_other}')
+
+        ids_to_filter_out = [elem.get('id') for elem in list(other)]
+
+        new_value = [elem for elem in list(deepcopy(self)) if elem.get('id') not in ids_to_filter_out]
+
+        new_object = self.__class__().from_list(new_value)
+
+        return new_object
+
+    
+    def __isub__(self, other):
+        """
+        __isub__ is used to filter out the elements in the self.value that exists in other.value inplace.
+        """
+        
+        new_object = self - other
+        self.value = new_object.value
+        return self
\ No newline at end of file
